/*!
    @header BTImage.h
    @project SMKBitmapTools
    @author Alexey Navrotsky
    @creation_date 01/19/2007
    @copyright Copyright (c) 2007 The Software MacKiev Company. All rights reserved.
*/

#import <Foundation/NSObject.h>
#import <Foundation/NSGeometry.h>
#import <ApplicationServices/../Frameworks/CoreGraphics.framework/Headers/CGGeometry.h>
#import <ApplicationServices/../Frameworks/CoreGraphics.framework/Headers/CGColorSpace.h>
#import <ApplicationServices/../Frameworks/CoreGraphics.framework/Headers/CGImage.h>
#import <ApplicationServices/../Frameworks/CoreGraphics.framework/Headers/CGPDFDocument.h>
#import <ApplicationServices/../Frameworks/CoreGraphics.framework/Headers/CGContext.h>
#import <ApplicationServices/../Frameworks/ColorSync.framework/Headers/CMTypes.h>
#import <SMKCGExtensions/CGEFloat.h>
#import <SMKBitmapTools/BTNSObjCRuntime.h>

@protocol BTImageUndoing, BTImageUndoingDelegate;
@class NSString;
@class NSGraphicsContext, NSOpenGLContext;
@class CIContext, CIImage, CIImageAccumulator;

enum
{ // Values 0 and less are reserved.
    BTImageFormatUnknown = 0,
    BTImageFormatARGB8888Premultiplied = -1
};

typedef NSInteger BTImageFormat;

/*!
    @protocol BTImage
*/
@protocol BTImage<NSObject>

- (CGSize)imageSize;

@end

/*!
    @protocol BTMutableImage
*/
@protocol BTMutableImage<BTImage>

// Result may be nil: image does not provide undoer.
- (id<BTImageUndoing>)createImageUndoerWithDelegate:(id<BTImageUndoingDelegate>)imageUndoerDelegate;

@end

#pragma mark -
#pragma mark *** Image Reading Protocols ***
#pragma mark -

/*!
    @protocol BTRawImage
*/
@protocol BTRawImage<BTImage>

- (const void*)imageBytes;
- (size_t)imageBytesPerRow;
- (BTImageFormat)imageFormat;
- (CMProfileRef)imageColorProfile; // Result may be nil.
- (const CGFloat*)imageDecode; // Result may be nil.

@end

/*!
    @protocol BTCGImage
*/
@protocol BTCGImage<BTImage>

- (void)drawInCGContext:(CGContextRef)context inRect:(CGRect)rect;
- (void)drawInCGContext:(CGContextRef)context atPoint:(CGPoint)point;

// *** Optional Routines (may always return nil) ***

- (CGPDFDocumentRef)coreGraphicsPDFDocument;

- (CGColorSpaceRef)coreGraphicsColorSpace;
- (CGImageRef)coreGraphicsImage;

@end

/*!
    @protocol BTCocoaImage
*/
@protocol BTCocoaImage<BTImage>

- (void)drawInCurrentNSGraphicsContextInRect:(NSRect)rect;
- (void)drawInCurrentNSGraphicsContextAtPoint:(NSPoint)point;

@end

/*!
    @protocol BTCIImage
*/
@protocol BTCIImage<BTImage>

- (CIImage*)CIImage;

@end

#pragma mark -
#pragma mark *** Image Writing Protocols ***
#pragma mark -

/*!
    @protocol BTRawMutableImage
*/
@protocol BTRawMutableImage<BTMutableImage>

- (void)beginRawEditing;

// These should only be sent between `-beginRawEditing' & `-endRawEditing'.
- (void*)mutableImageBytes;
- (size_t)imageBytesPerRow;
- (BTImageFormat)imageFormat;
- (CMProfileRef)imageColorProfile; // Result may be nil.
- (const CGFloat*)imageDecode; // Result may be nil.

- (void)endRawEditing;

@end

/*!
    @protocol BTCGMutableImage
*/
@protocol BTCGMutableImage<BTMutableImage>

- (void)beginCGEditing; // Also saves graphics state.

- (CGContextRef)imageCGContext; // Should only be sent between `-beginCGEditing' & `-endCGEditing'.

- (void)endCGEditing; // Also restores graphics state.

// *** Optional Routines (may always return nil) ***

- (CGColorSpaceRef)coreGraphicsColorSpace;

@end

/*!
    @protocol BTCocoaMutableImage
*/
@protocol BTCocoaMutableImage<BTMutableImage>

- (void)beginCocoaEditing; // Also saves graphics state.

- (void)endCocoaEditing; // Also restores graphics state.

@end

/*!
    @protocol BTCIMutableImage
*/
@protocol BTCIMutableImage<BTMutableImage>

- (void)beginCIEditing;

- (CIContext*)imageCIContext; // Should only be sent between `-beginCIEditing' & `-endCIEditing'.

- (void)endCIEditing;

@end

/*!
    @protocol BTCIAccumulableImage
*/
@protocol BTCIAccumulableImage<BTMutableImage>

- (void)beginCIAccumulableEditing;

- (CIImageAccumulator*)imageCIAccumulator; // Should only be sent between `-beginCIAccumulableEditing' & `-endCIAccumulableEditing'.

- (void)endCIAccumulableEditing;

@end

/*!
    @protocol BTNSOpenGLMutableImage
*/
@protocol BTNSOpenGLMutableImage<BTMutableImage>

- (void)beginOpenGLEditing;

- (NSOpenGLContext*)imageNSOpenGLContext; // Should only be sent between `-beginOpenGLEditing' & `-endOpenGLEditing'.

- (void)endOpenGLEditing;

@end

#pragma mark -
#pragma mark *** Mutable Image Undo Support Protocols ***
#pragma mark -

/*!
    @protocol BTImageUndoing
    @abstract Performs undo / redo operations for a mutable image.
    @discussion All operations with undo manager are delegated for better flexibility and modularity.
*/
@protocol BTImageUndoing<NSObject>

- (id<BTMutableImage>)image;
- (id<BTImageUndoingDelegate>)delegate;

- (void)registerUndoInRect:(CGRect)imageRect;

// Begin / end undo grouping of image changes. These may be sent from the same undo group only and may not be nested.
- (void)beginUndoGrouping;
- (void)endUndoGrouping;

@end

/*!
    @protocol BTImageUndoingDelegate
*/
@protocol BTImageUndoingDelegate

- (void)registerUndoForImageUndoer:(id<BTImageUndoing>)undoer withTarget:(id)target selector:(SEL)selector object:(id)object;
- (id)prepareUndoForImageUndoer:(id<BTImageUndoing>)undoer withInvocationTarget:(id)target;

- (void)imageUndoer:(id<BTImageUndoing>)undoer willChangeImageInRect:(CGRect)imageRect;
- (void)imageUndoer:(id<BTImageUndoing>)undoer didChangeImageInRect:(CGRect)imageRect;

@end
