/*!
    @header BTLayoutGuiding.h
    @project SMKBitmapTools
    @author Alexey Navrotsky
    @creation_date 2007/09/29
    @copyright Copyright (c) 2007 The Software MacKiev Company. All rights reserved.
    @abstract Interface for BTLayoutGuidingContext.
*/

#import <Foundation/NSGeometry.h>
#import <Foundation/NSObject.h>
#import <ApplicationServices/../Frameworks/CoreGraphics.framework/Headers/CGAffineTransform.h>
#import <SMKCGExtensions/CGEFloat.h>
#import <SMKCGExtensions/CGELine.h>

@class NSArray, NSDictionary;
@class NSEvent;

/*!
    @protocol BTLayoutGuidingContext
*/
@protocol BTLayoutGuidingContext

// `viewPointSnapSources' is array of id<BTPointSnapSource, BTPointToPointReflecting>. It may be nil.
// `viewSizeSnapSources' is array of id<BTSizeSnapSource, BTSizeToPointReflecting>. It may be nil.
// `event' may be nil.
// `options' may be nil.
// `snappedReflectedPointPtr' may be nil.
// `snapDistancePtr' may be nil.
// `appliedRulesPtr' may be nil.
- (BOOL)snapPoints:(NSArray*)viewPointSnapSources sizes:(NSArray*)viewSizeSnapSources withEvent:(NSEvent*)event options:(NSDictionary*)options snappedReflectedPoint:(CGPoint*)snappedReflectedPointPtr snapDistance:(CGFloat*)snapDistancePtr appliedRules:(NSArray**)appliedRulesPtr;

// `appliedRules' may be nil.
- (void)showAppliedRules:(NSArray*)appliedRules;

@end

/*!
    @protocol BTPointSnapSource
*/
@protocol BTPointSnapSource<NSObject>

- (CGPoint)point;

// ±INFINITY is valid value for snap bounds.
- (CGFloat)minXBound;
- (CGFloat)maxXBound;
- (CGFloat)minYBound;
- (CGFloat)maxYBound;

// Undefined line (f.e. CGELineUndefined) is valid value for trajectory line: means no trajectory.
- (CGELine)trajectoryLine;

// Result may be nil.
- (NSDictionary*)options;

@end

/*!
    @protocol BTSizeSnapSource
*/
@protocol BTSizeSnapSource<NSObject>

- (CGSize)size;

- (CGSize)minSize;
- (CGSize)maxSize; // +INFINITY is valid value for each dimension of max size.

// Proportion is width divided by height. Zero is valid value for proportion factor: means no proportion.
- (CGFloat)proportionFactor;

// Affine transform to show size (as result of applied rules): size should be shown inside parallelogram {size, show transform}.
- (CGAffineTransform)showTransform;

// Result may be nil.
- (NSDictionary*)options;

@end

/*!
    @protocol BTPointToPointReflecting
*/
@protocol BTPointToPointReflecting

- (CGPoint)reflectedPointForPoint:(CGPoint)point;

@end

/*!
    @protocol BTSizeToPointReflecting
*/
@protocol BTSizeToPointReflecting

- (CGPoint)reflectedPointForSize:(CGSize)size;

@end

#pragma mark -
#pragma mark *** Base Snap Source Classes ***
#pragma mark -

/*!
    @class BTPointSnapSource
*/
@interface BTPointSnapSource : NSObject<BTPointSnapSource>
{
@private
    CGPoint _point;
    CGFloat _minXBound;
    CGFloat _maxXBound;
    CGFloat _minYBound;
    CGFloat _maxYBound;
    CGELine _trajectoryLine;
    NSDictionary* _options;
}

- (id)initWithPoint:(CGPoint)point minXBound:(CGFloat)minXBound maxXBound:(CGFloat)maxXBound minYBound:(CGFloat)minYBound maxYBound:(CGFloat)maxYBound trajectoryLine:(CGELine)trajectoryLine options:(NSDictionary*)options; // Designated initializer.

@end

/*!
    @class BTSizeSnapSource
*/
@interface BTSizeSnapSource : NSObject<BTSizeSnapSource>
{
@private
    CGSize _size;
    CGSize _minSize;
    CGSize _maxSize;
    CGFloat _proportionFactor;
    CGAffineTransform _showTransform;
    NSDictionary* _options;
}

- (id)initWithSize:(CGSize)size minSize:(CGSize)minSize maxSize:(CGSize)maxSize proportionFactor:(CGFloat)proportionFactor showTransform:(CGAffineTransform)showTransform options:(NSDictionary*)options; // Designated initializer.

@end

#pragma mark -
#pragma mark *** Concrete Snap Source Classes ***
#pragma mark -

/*!
    @class BTPointToPointSnapSource
*/
@interface BTPointToPointSnapSource : BTPointSnapSource<BTPointToPointReflecting>
{
@private
    CGAffineTransform _reflectingTransform;
}

+ (id)sourceWithPoint:(CGPoint)point minXBound:(CGFloat)minXBound maxXBound:(CGFloat)maxXBound minYBound:(CGFloat)minYBound maxYBound:(CGFloat)maxYBound trajectoryLine:(CGELine)trajectoryLine options:(NSDictionary*)options reflectingTransform:(CGAffineTransform)reflectingTransform;

- (id)initWithPoint:(CGPoint)point minXBound:(CGFloat)minXBound maxXBound:(CGFloat)maxXBound minYBound:(CGFloat)minYBound maxYBound:(CGFloat)maxYBound trajectoryLine:(CGELine)trajectoryLine options:(NSDictionary*)options reflectingTransform:(CGAffineTransform)reflectingTransform; // Designated initializer.

@end

/*!
    @class BTSizeToPointSnapSource
*/
@interface BTSizeToPointSnapSource : BTSizeSnapSource<BTSizeToPointReflecting>
{
@private
    CGPoint _basePoint;
    CGPoint _resizingCenter;
    CGAffineTransform _reflectingTransform;
}

+ (id)sourceWithSize:(CGSize)size minSize:(CGSize)minSize maxSize:(CGSize)maxSize proportionFactor:(CGFloat)proportionFactor showTransform:(CGAffineTransform)showTransform options:(NSDictionary*)options basePoint:(CGPoint)basePoint resizingCenter:(CGPoint)resizingCenter reflectingTransform:(CGAffineTransform)reflectingTransform;

- (id)initWithSize:(CGSize)size minSize:(CGSize)minSize maxSize:(CGSize)maxSize proportionFactor:(CGFloat)proportionFactor showTransform:(CGAffineTransform)showTransform options:(NSDictionary*)options basePoint:(CGPoint)basePoint resizingCenter:(CGPoint)resizingCenter reflectingTransform:(CGAffineTransform)reflectingTransform; // Designated initializer.

@end

#pragma mark *** Point Snap Source Options ***

/*!
    @const BTPointSnapSourceOptionSnapAsDevicePixelCenter
    @abstract BTPointSnapSourceOptionSnapAsDevicePixelCenter
                Value: NSNumber with BOOL.
    @discussion If value is YES, then point should be snapped as center of a device pixel (exact rules for snapping as center of a device pixel are defined by layout guiding context though). Absence of this value is the same as if value is NO. Note: if value is YES then point in the source should be centered on a device pixel.
*/
#define BTPointSnapSourceOptionSnapAsDevicePixelCenter @"BTPointSnapSourceOptionSnapAsDevicePixelCenter"
