/*!
    @header BTTool.h
    @project SMKBitmapTools
    @author Alexey Navrotsky
    @creation_date 12/08/2006
    @copyright Copyright (c) 2006 The Software MacKiev Company. All rights reserved.
*/

#import <Foundation/NSObject.h>
#import <Foundation/NSGeometry.h>
#import <ApplicationServices/../Frameworks/CoreGraphics.framework/Headers/CGAffineTransform.h>
#import <ApplicationServices/../Frameworks/CoreGraphics.framework/Headers/CGContext.h>
#import <SMKBitmapTools/BTNSObjCRuntime.h>
#import <SMKBitmapTools/BTImage.h>


@protocol BTToolDelegate, BTImageViewTransformer, BTMouseTracker, BTLayoutGuidingContext;
@class NSArray, NSSet, NSMutableSet, NSDictionary, NSMutableDictionary, NSUndoManager;
@class NSView, NSEvent, NSCursor, NSPasteboard;


enum
{
    BTToolOptionAlwaysOn = 0,
    BTToolOptionAlwaysOff = NSUIntegerMax
};

// Valid tool options are as follows:
//      - BTToolOptionAlwaysOn: option is always active;
//      - BTToolOptionAlwaysOff: option is always inactive;
//      - a value constructed from device-independent bits of event modifier flags (see NSEvent.h): activating modifiers are included as is (i.e. in upper 16 bits) and deactivating modifiers are included with offset to right by 16 bits (i.e. in lower 16 bits).
// Examples:
//      - (NSCommandKeyMask | NSShiftKeyMask): option is active while both Command & Shift modifiers are pressed;
//      - (NSAlternateKeyMask >> 16): option is active while Option modifier is not pressed;
//      - (NSCommandKeyMask | (NSShiftKeyMask >> 16)): option is active while Command modifier is pressed & Shift modifiers is not pressed.
typedef NSUInteger BTToolOption;

FOUNDATION_STATIC_INLINE BOOL BTToolOptionIsActiveWithModifierFlags(BTToolOption toolOption, NSUInteger modifierFlags);

extern const id BTUndefinedValue;

#pragma mark -


/*!
    @class BTTool
    @discussion Stage cycle: BTWaitingToolStage.
*/
@interface BTTool : NSObject<BTImageUndoingDelegate>
{
@private
    id<BTImage> _workImage;
    id<BTImageUndoing> _workImageUndoer;
    NSMutableDictionary* _undoProxies;
    NSUInteger _undoRegistrationEventNotingCounter;
    NSPasteboard* _workPasteboard;
    CGContextRef _referenceCGContext;
    NSDictionary* _generalKeyBindings;
    void* _pointer;
    
    BOOL _isActive : 1;
    BOOL _isViewInvalidationEnabled : 1;
    BOOL _isKeyEventHandled : 1;
    BOOL _isWorkPasteboardInitialized : 1;
    BOOL _areGeneralKeyBindingsInitialized : 1;
    BOOL _isWorkImageUndoGroupingBegun : 1;
    BOOL _isWorkImageUndoGroupingScheduled : 1;
    BOOL _isWorkImageBeingChanged : 1;
    BOOL _isWorkImageBeingReplaced : 1;
    
@protected
    id<BTToolDelegate> _delegate; // Weak reference to the active state delegate. Should not be changed by subclasses.
    NSView* _view; // Weak reference to the active state view. Should not be changed by subclasses.
    
    // Weak (non-retained) reference to the current subtool (if any).
    // Many BTTool's implementations of event & input handling, drawing and change... methods forward processing to the subtool. All unknown selectors are forwarded to the subtool. `- (BOOL)respondsToSelector:(SEL)selector' is forwarded to the subtool for unknown selectors as well.
    BTTool* _currentSubtool;
}

// *** Image Accessors ***

- (id<BTImage>)workImage;

// Returns YES if `image' is set successfully.
- (BOOL)setWorkImage:(id<BTImage>)image;
- (BOOL)setWorkImage:(id<BTImage>)image withEffectiveChangeInRect:(CGRect)imageRect;

// Return YES if `image' will be accepted by the receiver's `setWorkImage:'.
// BTTool's implementation checks for conformance of `image' to required image protocols.
// Note: this message is sent from "-setWorkImage..." methods.
- (BOOL)verifyImage:(id<BTImage>)image;

// Returns nil-terminated array of image protocols (protocols conformed to BTImage protocol) which are required for the receiver.
- (Protocol* const*)requiredImageProtocols;

// Returns nil-terminated array of image protocols (protocols conformed to BTImage protocol) which may be used by the receiver for optimization purposes.
- (Protocol* const*)optionalImageProtocols;

// *** Inputs & Outputs ***

// Returns an array containing the names of all inputs in the tool.
- (NSArray*)inputKeys;

// Returns an array containing the names of all outputs in the tool.
- (NSArray*)outputKeys;

- (id)valueForInputKey:(NSString*)inputKey;
- (void)setValue:(id)value forInputKey:(NSString*)inputKey;

- (NSDictionary*)inputs;
- (void)setInputs:(NSDictionary*)inputsDictionary;

// Returns default value for `inputKey' or BTUndefinedValue if there is no one.
// BTTool's implementation composes key for default value by uppercasing first letter of `inputKey' and adding "default" to its beginning (f.e. "cursor" --> "defaultCursor") & uses KVC to retrieve the value.
- (id)defaultValueForInputKey:(NSString*)inputKey;

- (id)valueForOutputKey:(NSString*)outputKey;

// *** Setting Defaults ***

// Sets all inputs to their default values (where default values are defined, other inputs are left as-is).
- (void)setDefaults;

// *** Concrete Inputs & Outputs ***

// Returns default general key bindings.
// BTTool's implementation attempts to load receiver's default general key bindings from the dictionary file which is of the same format as system key bindings, is named the same as tool class, has extension ".dict" and is located in the "DefaultKeyBindings" subfolder of the framework's resource folder.
- (NSDictionary*)defaultGeneralKeyBindings;

// General key bindings in effect for all receiver's states/modes.
- (NSDictionary*)generalKeyBindings;
- (void)setGeneralKeyBindings:(NSDictionary*)generalKeyBindings;

// *** Undo Management ***

// BTTool's implementation returns empty set.
- (NSSet*)allUndoCategories;

// Note: it is owner's responsibility to remove the receiver from undo manager(s) before its deallocation.
- (void)removeAllUndoActionsFromManager:(NSUndoManager*)undoManager category:(NSString*)undoCategory;
- (void)removeAllUndoActionsFromManager:(NSUndoManager*)undoManager;

// *** Other Settings ***

// Default work pasteboard is NSGeneralPboard.
- (NSPasteboard*)workPasteboard;

// `pasteboard' may be nil.
- (void)setWorkPasteboard:(NSPasteboard*)pasteboard;

// Result may be nil.
- (CGContextRef)referenceCGContext;

// Sets reference CoreGraphics context which may be used for optimization purposes (f.e. it may be used as reference context for CGLayers).
// `context' may be nil.
- (void)setReferenceCGContext:(CGContextRef)context;

// View invalidation is enabled by default.
- (BOOL)isViewInvalidationEnabled;
- (void)setViewInvalidationEnabled:(BOOL)flag;

// *** Activation & Deactivation ***

- (BOOL)isActive;

- (BOOL)activateWithDelegate:(id<BTToolDelegate>)toolDelegate inView:(NSView*)toolView;

// Deactivation may be normal (tool will fulfill all owner's & user's requests) & abortive (tool may do not fulfill all owner's & user's requests, tool's internal operations may be aborted). If deactivation occurs from within `-tool:didFinalizeStage:withEvent:nextStage:' then it is normal (regardless of current stage); otherwise, deactivation type depends on current stage of the receiver (see deactivation type in stage's description).
// This method may not be sent from within any delegate method (including sub-delegate methods - methods of protocols BTImageViewTransformer, BTMouseTracker, BTLayoutGuidingContext) except `-tool:didFinalizeStage:withEvent:nextStage:'.
- (void)deactivate;

// *** Active State Info ***

- (id<BTToolDelegate>)delegate;
- (NSView*)view;

- (NSString*)currentStage;

// BTTool's implementation forwards the message to the current subtool.
- (BOOL)isTracking;

// *** Event Handling (Active State Only) ***

// It is sender's responsibility to check if event location inside tool view visible bounds.
// It is sender's responsibility to update current cursor for the receiver.
// BTTool's implementation forwards the message to the current subtool.
- (NSCursor*)cursorForPoint:(NSPoint)viewPoint modifierFlags:(NSUInteger)modifierFlags;

// Dispatches `event' via appropriate method of the receiver. Returns YES if the receiver did handle `event'.
// Note: BTTool's implementation does not forward to the current subtool, concreate event processing methods do forward.
- (BOOL)sendEvent:(NSEvent*)event;

// BTTool's implementations forward the message to the current subtool.
- (BOOL)mouseDown:(NSEvent*)event;
- (BOOL)mouseUp:(NSEvent*)event;
- (BOOL)mouseDragged:(NSEvent*)event;
- (BOOL)mouseMoved:(NSEvent*)event;
- (BOOL)scrollWheel:(NSEvent*)event;

// BTTool's implementation dose the following: sends `- (BOOL)rawKeyDown:(NSEvent*)event' to self for the opportunity to handle the event as raw key down; if the event is not handled as raw key down, the receiver tries to interpret the event using its key bindings (`[self keyBindings]'); if the event is not interpreted using the receiver's key bindings, the reseiver's view is given the opportunity to interpret the event using `- (void)interpretCurrentKeyEventAsCommand:(SEL*)commandSelectorPtr string:(NSString**)insertStringPtr'; if the reseiver's view does not implement `- (void)interpretCurrentKeyEventAsCommand:(SEL*)commandSelectorPtr string:(NSString**)insertStringPtr', Cocoa’s text input management system is asked to interpret the event.
// Note: Though BTTool's implementation of this method does not forward it to the current subtool directly, BTTool's implementations of `- (BOOL)rawKeyDown:(NSEvent*)event' & input processing methods do forward.
- (BOOL)keyDown:(NSEvent*)event;

// BTTool's implementation forwards the message to the current subtool.
- (BOOL)keyUp:(NSEvent*)event;

// BTTool's implementation forwards the message to the current subtool.
- (BOOL)flagsChanged:(NSEvent*)event;

// *** Handling of Input (Active State Only) ***

// Returns effective key binding dictionary (for current state/mode).
// BTTool's implementation returns merge of receiver's general key bindings with subtool's general key bindings (if both dictionaries have the same key, the receiver's one is taken).
- (NSDictionary*)keyBindings;

// BTTool's implementation composes validation selector as "- (BOOL)validateAction_%action_name%" (f.e. "copy:" --> "validateAction_copy") and attempts to perform it using `-performValidationSelector:'. If any error occurs, returns NO.
// Note: Though BTTool's implementation does not forward the validation selector to the current subtool directly, the subtool participates in validation via standard message forwarding mechanism.
- (BOOL)validateAction:(SEL)action;

// Attempts to perform `validationSelector'. If attempt succeeds, returns the validate message's result; otherwise, returns YES. If any error occurs, returns NO.
// `validationSelector' is expected to have no arguments and to return BOOL.
// Note: Though BTTool's implementation does not forward the validation selector to the current subtool directly, the subtool participates in validation via standard message forwarding mechanism.
- (BOOL)performValidationSelector:(SEL)validationSelector;

// Attempts to perform `selector'. Returns YES on success.
// Note: Though BTTool's implementation does not forward the message to the current subtool directly, the subtool participates in doing commands via standard message forwarding mechanism.
- (BOOL)doCommandBySelector:(SEL)selector;

// Attempts to insert text `insertString'. Returns YES if the insertion of the text is accepted.
// BTTool's implementation forwards the message to the current subtool.
- (BOOL)insertText:(NSString*)insertString;

// *** High-Level Drawing (Active State Only) ***

// Draws receiver's feedback (all feedback layers).
// BTTool's implementation simply draws (using `- (void)performDrawRect:(NSRect)rect ofActiveFeedbackLayer:(NSString*)feedbackLayer') all feedback layers returned by `- (NSArray*)activeFeedbackLayersForDrawing'.
- (void)drawRect:(NSRect)rect;

// *** Low-Level Drawing (Active State Only): Feedback Layers ***
// Note: feedback layers may change as receiver changes its (public or private) mode(s) in response to user events or setting inputs.
// BTTool's implementations use following methods as primitives for implementation of feedback layer drawing (though they are not mandatory to override in subclasses since a subclass may have no drawing feedback or may forward it to its subtool which is already done by BTTool's implementations): `- (NSArray*)activeFeedbackLayersForDrawing' and "- (void)draw_%feedback_layer%_Rect:(NSRect)rect" methods (see `- (void)performDrawRect:(NSRect)rect ofActiveFeedbackLayer:(NSString*)feedbackLayer' for details).

// BTTool implements this using `- (NSArray*)activeFeedbackLayersForDrawing'.
- (NSSet*)activeFeedbackLayers;

// Returns YES if `feedbackLayer' exists and is active now.
// BTTool implements this using `- (NSSet*)activeFeedbackLayers'.
- (BOOL)isFeedbackLayerActive:(NSString*)feedbackLayer;

// Returns active feedback layers ordered for drawing.
// BTTool's implementation forwards the message to the current subtool if it's present.
- (NSArray*)activeFeedbackLayersForDrawing;

// Drawing order (if applicable) is defined by the receiver (as `- (NSArray*)activeFeedbackLayersForDrawing' returns).
// Note: only existing and active feedback layers are drawn, so it is safe to pass in any string(s) as feedback layer(s).
// BTTool implements these using `- (void)performDrawRect:(NSRect)rect ofActiveFeedbackLayer:(NSString*)feedbackLayer'.
- (void)drawRect:(NSRect)rect ofActiveFeedbackLayer:(NSString*)feedbackLayer;
- (void)drawRect:(NSRect)rect ofActiveFeedbackLayers:(NSSet*)feedbackLayers;
- (void)drawRect:(NSRect)rect ofActiveFeedbackLayersExcludingLayer:(NSString*)feedbackLayer;
- (void)drawRect:(NSRect)rect ofActiveFeedbackLayersExcludingLayers:(NSSet*)feedbackLayers;

// *** Convenience / Utility Methods (Active State Only) ***

// Registers / prepares undo in category `undoCategory'.
- (void)registerUndoInManager:(NSUndoManager*)undoManager category:(NSString*)undoCategory withTarget:(id)target selector:(SEL)selector object:(id)object;
- (id)prepareUndoInManager:(NSUndoManager*)undoManager category:(NSString*)undoCategory withInvocationTarget:(id)target;

// Blocks further undo action name setting with nested `-beginUndoAction' & `-endUndoActionWithName:' pair until `-endUndoActionWithName:' is sent. Begins noting of event with name BTToolUndoRegistrationEventName (see NSThreadBTEventNoting.h for details).
- (void)beginUndoAction;

// Sets `undoActionName' (using `-setUndoActionName:') if undo was registered (via `-registerUndoInManager:category:withTarget:selector:object:' or `-prepareUndoInManager:category:withInvocationTarget:' or `-[NSThread BT_noteEventWithName:BTToolUndoRegistrationEventName]') after pairing `-beginUndoAction'. Ends noting of event with name BTToolUndoRegistrationEventName (see NSThreadBTEventNoting.h for details).
// `undoActionName' may be nil: no undo action name is set.
- (void)endUndoActionWithName:(NSString*)undoActionName;

// Sets undo action name if the delegate returns valid undo manger for the action (`-tool:willSetUndoActionName:'), then `undoActionName' is localized using "Undo" table of the framework (com.mackiev.framework.SMKBitmapTools) and is set into the undo manager.
- (void)setUndoActionName:(NSString*)undoActionName;

// Registers undo of work image changes if delegate permits undo registration in category "BTToolWorkImageUndoCategory".
// Work image is expected to be mutable and capable of creating an undoer.
- (void)registerWorkImageUndoInRect:(CGRect)imageRect;

// Begins / ends undo grouping of work image changes if delegate permits undo registration in category "BTToolWorkImageUndoCategory".
// Work image is expected to be mutable and capable of creating an undoer.
// These may be sent from the same undo group only and may not be nested.
// Note: nothing is done if no work image undo is registered between begin / end messages.
- (void)beginWorkImageUndoGrouping;
- (void)endWorkImageUndoGrouping;
- (BOOL)isGroupingWorkImageUndo;

// If delegate is not nil and has image-view transformer then forwards to it (transformer), otherwise assumes no transformation. 
- (CGAffineTransform)viewToImageTransform;
- (NSPoint)convertViewPointToImagePoint:(NSPoint)viewPoint;
- (NSRect)convertViewRectToImageRect:(NSRect)viewRect;
- (CGAffineTransform)imageToViewTransform;
- (NSPoint)convertImagePointToViewPoint:(NSPoint)imagePoint;
- (NSRect)convertImageRectToViewRect:(NSRect)imageRect;

// Point for conversion is center of the window pixel pointed by the mouse.
- (NSPoint)eventLocationInView:(NSEvent*)event;
- (NSPoint)eventLocationInImage:(NSEvent*)event;

// If delegate is not nil and has layout guiding context then forwards to it (context), otherwise assumes no snapping. 
// `event' may be nil.
// `options' may be nil.
- (BOOL)snapPoint:(CGPoint*)viewPointPtr asDevicePixelCenter:(BOOL)snapAsDevicePixelCenterFlag withinBoundsWithMinX:(CGFloat)minXBound maxX:(CGFloat)maxXBound minY:(CGFloat)minYBound maxY:(CGFloat)maxYBound alongTrajectoryLine:(CGELine)trajectoryLine withEvent:(NSEvent*)event options:(NSDictionary*)options;
- (BOOL)snapPoint:(CGPoint)viewPoint asDevicePixelCenter:(BOOL)snapAsDevicePixelCenterFlag withinBoundsWithMinX:(CGFloat)minXBound maxX:(CGFloat)maxXBound minY:(CGFloat)minYBound maxY:(CGFloat)maxYBound alongTrajectoryLine:(CGELine)trajectoryLine withEvent:(NSEvent*)event options:(NSDictionary*)options snappedPoint:(CGPoint*)snappedPointPtr snapDistance:(CGFloat*)snapDistancePtr appliedRules:(NSArray**)appliedRulesPtr;

// Applies mode & layout guiding rules for a movement operation. Returns guided end point.
// `centerOnDevicePixelFlag' defines whether to center end point on device pixel for both mode guiding & layout guiding. Note: if `centerOnDevicePixelFlag' is YES then both `viewStartPoint' & `viewEndPoint' should be centered on device pixels.
// `guidingModePtr' may be nil: means 2D mode. If it is not nil then it is in-out with values as follows:
//      - 0: 2D mode, unchanged;
//      - 1: 1D horizontal mode, unchanged;
//      - 2: 1D vertical mode, unchanged;
//      - 3: 1D diagonal 45 degrees mode, unchanged;
//      - 4: 1D diagonal 135 degrees mode, unchanged;
//      - any other value: 1D undefined mode, if `viewStartPoint' & `viewEndPoint' are not equal then changed to concrete 1D mode (1-4) which was applied.
// `event' may be nil.
- (CGPoint)guideMovementFromPoint:(CGPoint)viewStartPoint toPoint:(CGPoint)viewEndPoint centeringOnDevicePixel:(BOOL)centerOnDevicePixelFlag inMode:(unsigned char*)guidingModePtr withEvent:(NSEvent*)event;

// Applies mode & layout guiding rules for a movement operation. Returns guided end point.
// `guideAsDevicePixelCentersFlag' defines whether to center input points on device pixel for both mode guiding & layout guiding.
// `centerOnDevicePixelFlag' defines whether to center end point on device pixel for mode guiding if no layout guiding occurred. It may not be YES if `guideAsDevicePixelCentersFlag' is YES. Note: if `centerOnDevicePixelFlag' is YES then both `viewStartPoint' & `viewEndPoint' should be centered on device pixels.
// `guidingModePtr' may be nil: means 2D mode. If it is not nil then it is in-out with values as follows:
//      - 0: 2D mode, unchanged;
//      - 1: 1D horizontal mode, unchanged;
//      - 2: 1D vertical mode, unchanged;
//      - 3: 1D diagonal 45 degrees mode, unchanged;
//      - 4: 1D diagonal 135 degrees mode, unchanged;
//      - any other value: 1D undefined mode, if `viewStartPoint' & `viewEndPoint' are not equal then changed to concrete 1D mode (1-4) which was applied.
// `event' may be nil.
- (CGPoint)guidePoints:(const CGPoint*)viewPoints count:(NSUInteger)pointCount withReflectingTransforms:(const CGAffineTransform*)reflectingTransforms asDevicePixelCenters:(BOOL)guideAsDevicePixelCentersFlag forMovementFromPoint:(CGPoint)viewStartPoint toPoint:(CGPoint)viewEndPoint centeringOnDevicePixel:(BOOL)centerOnDevicePixelFlag inMode:(unsigned char*)guidingModePtr withEvent:(NSEvent*)event;

// These notify the delegate via appropriate methods and register undo of work image changes if undo registration in category "BTToolWorkImageUndoCategory" is enabled.
- (CGRect)preEditWorkImage:(id<BTImage>)image; // Returns rect to be passed into `-postEditWorkImageInRect:'.
- (void)preEditWorkImage:(id<BTImage>)image inRect:(CGRect)imageRect;
- (void)postEditWorkImageInRect:(CGRect)imageRect;

// *** Protected Methods ***

// Do not send directly. Does actual change of the work image, no analysis if it should occur.
- (void)changeWorkImageTo:(id<BTImage>)image;

// Do not send directly. It is sent when actual change of work pasteboard occurs.
- (void)changeWorkPasteboardTo:(NSPasteboard*)pasteboard;

// Do not send directly. It is sent when actual change of reference CoreGraphics context occurs.
- (void)changeReferenceCGContextTo:(CGContextRef)context;

// Do not send directly. It is sent when actual change of view invalidation flag occurs.
- (void)changeViewInvalidationEnabledTo:(BOOL)flag;

// Do not send directly. Sent from `- (BOOL)keyDown:(NSEvent*)event' (see its description for more details).
// BTTool's implementation forwards the message to the current subtool.
- (BOOL)rawKeyDown:(NSEvent*)event;

// Do not send directly. Does actual deactivation, no analysis if it should occur.
- (void)performDeactivate;

// Do not send directly. Does drawing of `feedbackLayer', no analysis if the layer exists and active.
// BTTool's implementation composes drawing selector as "- (void)draw_%feedback_layer%_Rect:(NSRect)rect" (f.e. "BTSelectionFeedbackLayer" --> "draw_BTSelectionFeedbackLayer_Rect:") and attempts to perform it.
// Note: Though BTTool's implementation does not forward the message to the current subtool directly, the subtool participates in the drawing via standard message forwarding mechanism.
- (void)performDrawRect:(NSRect)rect ofActiveFeedbackLayer:(NSString*)feedbackLayer;

@end
#pragma mark @end
#pragma mark -


/*!
    @protocol BTToolDelegate
*/
@protocol BTToolDelegate

// `event' may be nil.
- (void)tool:(BTTool*)tool didUpdateCursorWithEvent:(NSEvent*)event;

- (void)toolWillBeginChangingImageContinuously:(BTTool*)tool;
- (void)toolDidEndChangingImageContinuously:(BTTool*)tool;

- (void)tool:(BTTool*)tool willChangeImageInRect:(NSRect)imageRect;
- (void)tool:(BTTool*)tool didChangeImageInRect:(NSRect)imageRect;

- (void)tool:(BTTool*)tool willReplaceImageWithEffectiveChangeInRect:(NSRect)imageRect;
- (void)tool:(BTTool*)tool didReplaceImageWithEffectiveChangeInRect:(NSRect)imageRect;

- (void)tool:(BTTool*)tool willUpdateOutputForKey:(NSString*)outputKey;
- (void)tool:(BTTool*)tool didUpdateOutputForKey:(NSString*)outputKey;

// Notifies the delegate that output for key `outputKey' is finalized: `tool' will not change it until the tool finalizes (with `-tool:didFinalizeWithEvent:') or is reactivated (the output may be changed through tool's inputs though).
// `event' may be nil.
- (void)tool:(BTTool*)tool didFinalizeOutputForKey:(NSString*)outputKey;

// Sent by `tool' upon finalizing its work stage `stage'. The tool should expect normal deactivation while sending this message.
// Returns whether `event' was handled by the receiver (this result may affect result of an event handling method of `tool').
// `event' may be nil.
// `nextStage' may be nil: `tool' finalizes its work cycle.
// Note: at time this message is sent `tool' is still in stage `stage', switching occurs after this method returns if `tool' remains active.
- (BOOL)tool:(BTTool*)tool didFinalizeStage:(NSString*)stage withEvent:(NSEvent*)event nextStage:(NSString*)nextStage;

// Result may be nil: no undo registration will occur.
- (NSUndoManager*)undoManagerForTool:(BTTool*)tool category:(NSString*)undoCategory;

- (void)tool:(BTTool*)tool willRegisterUndoInManager:(NSUndoManager*)undoManager category:(NSString*)undoCategory;

// Returns undo manager to set undo action name to.
// Result may be nil: undo action name will not be set (the delegate may localize and set it instead).
- (NSUndoManager*)tool:(BTTool*)tool willSetUndoActionName:(NSString*)undoActionName;

// Result may be nil: no transformation.
- (id<BTImageViewTransformer>)imageViewTransformerForTool:(BTTool*)tool;

// Returns delegate for mouse tracking: `tool' sends tracking messages to the delegate prior to processing them iteself. Result may be nil.
// See NSViewBTMouseTracking.h for details.
- (id<BTMouseTracker>)mouseTrackingDelegateForTool:(BTTool*)tool;

// Result may be nil.
// See BTLayoutGuiding.h for details.
- (id<BTLayoutGuidingContext>)layoutGuidingContextForTool:(BTTool*)tool;

@end
#pragma mark @end
#pragma mark -


/*!
    @protocol BTImageViewTransformer
*/
@protocol BTImageViewTransformer

- (CGAffineTransform)viewToImageTransform;
- (NSPoint)convertViewPointToImagePoint:(NSPoint)viewPoint;
- (NSRect)convertViewRectToImageRect:(NSRect)viewRect;

- (CGAffineTransform)imageToViewTransform;
- (NSPoint)convertImagePointToViewPoint:(NSPoint)imagePoint;
- (NSRect)convertImageRectToViewRect:(NSRect)imageRect;

@end
#pragma mark @end
#pragma mark -


#pragma mark *** Stages ***

/*!
    @const BTWaitingToolStage
    @abstract Tool is waiting for user input.
    @discussion Deactivation type: normal.
*/
#define BTWaitingToolStage @"BTWaitingToolStage"


#pragma mark *** Undo Categories ***

/*!
    @const BTToolWorkImageUndoCategory
    @abstract Category of changes to tool's work image.
    @discussion Undoable outputs: workImage.
*/
#define BTToolWorkImageUndoCategory @"BTToolWorkImageUndoCategory"


#pragma mark *** Undo Registartion Event ***

/*!
    @const BTToolUndoRegistrationEventName
    @abstract Name of bitmap tools' undo registration event.
    @discussion See NSThreadBTEventNoting.h for details.
*/
#define BTToolUndoRegistrationEventName @"BTToolUndoRegistrationEventName"


#pragma mark *** Inline Definitions ***

FOUNDATION_STATIC_INLINE BOOL BTToolOptionIsActiveWithModifierFlags(BTToolOption toolOption, NSUInteger modifierFlags)
{
    NSUInteger actvatingModifiers = toolOption & NSDeviceIndependentModifierFlagsMask;
    NSUInteger deactvatingModifiers = (toolOption << 16) & NSDeviceIndependentModifierFlagsMask;
    
    return ((modifierFlags & actvatingModifiers) == actvatingModifiers) && ((modifierFlags & deactvatingModifiers) == 0);
}
