/*!
    @header BTTrackingView.h
    @project SMKBitmapTools
    @author Alexey Navrotsky
    @creation_date 12/18/2006
    @copyright Copyright (c) 2006 The Software MacKiev Company. All rights reserved.
*/

#import <AppKit/NSView.h>
#import <SMKBitmapTools/BTNSObjCRuntime.h>

@class BTWeakObjectProxy, BTTrackingZone;

/*!
    @class BTTrackingView
    @discussion Requirements for trackig zones: the view & all its superviews should be configured to post bounds & frame change notifications.
*/
@interface BTTrackingView : NSView
{
@private
    BTWeakObjectProxy* _trackingViewProxy;
    NSMutableSet* _trackingZones;
    BOOL _isRegisteredForViewNotifications;
}

/*!
    @method addTrackingZoneWithRect:owner:userData:assumeInside:
    @abstract Establishes an autoresizing area of visible rect of the receiver for tracking mouse-entered and mouse-exited events within the receiver and returns a tracking zone object that identifies the tracking area.
    @param object The object that gets sent the event messages. It can be the receiver itself or some other object (such as an NSCursor or a custom drawing tool object), as long as it responds to both `mouseEntered:' and `mouseExited:'.
    @param assumeInsideFlag If YES, the first event will be generated when the cursor leaves the zone; if NO the first event will be generated when the cursor enters it. `flag' determines which event is sent first by indicating where the cursor is assumed to be at the time this method is invoked.
    @param postInitialMouseEnteredFlag If YES & `assumeInsideFlag' is YES then initial NSMouseEntered event is posted.
    @result A tracking zone object. It is owned by the receiver till discarded.
*/
- (BTTrackingZone*)addTrackingZoneForVisibleRectWithOwner:(id)object assumeInside:(BOOL)assumeInsideFlag postInitialMouseEntered:(BOOL)postInitialMouseEnteredFlag;

/*!
    @method addTrackingZoneWithRect:owner:userData:assumeInside:
    @abstract Establishes an autoresizing area for tracking mouse-entered and mouse-exited events within the receiver and returns a tracking zone object that identifies the tracking area.
    @param rect Rect that defines zone (autoresizing area) of the receiver for tracking mouse-entered and mouse-exited events. It is in view space & is adjusted according to `autoresizingMask' during change of bounds or frame of the receiver.
    @param autoresizingMask Autoresizing mask for zone rect. Constants are the same as for autoresizing mask of NSView.
    @param object The object that gets sent the event messages. It can be the receiver itself or some other object (such as an NSCursor or a custom drawing tool object), as long as it responds to both `mouseEntered:' and `mouseExited:'.
    @param clipToVisibleRectFlag If YES clips tracking area to visible rect.
    @param assumeInsideFlag If YES, the first event will be generated when the cursor leaves the zone; if NO the first event will be generated when the cursor enters it. `flag' determines which event is sent first by indicating where the cursor is assumed to be at the time this method is invoked.
    @param postInitialMouseEnteredFlag If YES & `assumeInsideFlag' is YES then initial NSMouseEntered event is posted.
    @result A tracking zone object. It is owned by the receiver till discarded.
*/
// UNDER CONSIDERATION (can be implemented if needed)
//- (BTTrackingZone*)addTrackingZoneForRect:(NSRect)rect withAutoresizingMask:(NSUInteger)autoresizingMask owner:(id)object clipToVisibleRect:(BOOL)clipToVisibleRectFlag assumeInside:(BOOL)assumeInsideFlag postInitialMouseEntered:(BOOL)postInitialMouseEnteredFlag;

@end

@interface BTTrackingZone : NSObject

- (BTTrackingView*)view;
- (id)owner;

- (BOOL)isActive;

// These are valid only if receiver is active.
- (NSTrackingRectTag)trackingRectTag;
- (NSRect)windowTrackingRect;

// Invalidates the tracking zone.
- (void)discard;

@end
