/*!
    @header CGEColor.h
    @project SMKCGExtensions
    @author Alexey Navrotsky
    @creation_date 2007/03/26
    @copyright Copyright (c) 2007 The Software MacKiev Company. All rights reserved.
    @discussion Note: CGEColorRef is not CF-type.
*/

#ifndef CGEColor_H_
#define CGEColor_H_

#include <ApplicationServices/../Frameworks/CoreGraphics.framework/Headers/CGColor.h>
#include <ApplicationServices/../Frameworks/CoreGraphics.framework/Headers/CGColorSpace.h>
#include <ApplicationServices/../Frameworks/CoreGraphics.framework/Headers/CGPattern.h>
#include <ApplicationServices/../Frameworks/CoreGraphics.framework/Headers/CGFunction.h>
#include <ApplicationServices/../Frameworks/CoreGraphics.framework/Headers/CGImage.h>
#include <ApplicationServices/../Frameworks/CoreGraphics.framework/Headers/CGPDFDocument.h>
#include <ApplicationServices/../Frameworks/CoreGraphics.framework/Headers/CGContext.h>
#include <ApplicationServices/../Frameworks/CoreGraphics.framework/Headers/CGGeometry.h>
#include <SMKCGExtensions/CGEFloat.h>

CG_EXTERN_C_BEGIN

// *** Color Conversions ***

// `huePtr', `saturationPtr', `valuePtr' may be nil.
CG_EXTERN void CGEColorConvertRGBToHSV(CGFloat red, CGFloat green, CGFloat blue, CGFloat* huePtr, CGFloat* saturationPtr, CGFloat* valuePtr);

// `redPtr', `greenPtr', `bluePtr' may be nil.
CG_EXTERN void CGEColorConvertHSVToRGB(CGFloat hue, CGFloat saturation, CGFloat value, CGFloat* redPtr, CGFloat* greenPtr, CGFloat* bluePtr);

// *** CGEColor ***
// CGEColor may be one of three types:
//      - CGColor-based: behaves the same as underlying CGColor;
//      - user pattern: unlike pattern CGColor, user pattern CGEColor operates in current (transformed) user space (i.e. it is affected by CGContext's CTM); user pattern is tiled starting from color bounds' origin and is unaffected by CGContext's pattern phase;
//      - gradient: unlike CGShading, gradient CGEColor is defined in abstract space {{0.0, 0.0}, {1.0, 1.0}} which is mapped to color bounds at drawing time.
// Note: using of CGTransparencyLayer may change context's CTM which leads to incorrect positioning of user pattern origin.

typedef struct CGEColor* CGEColorRef;

typedef void (*CGEColorDrawUserPatternCallback)(void* info, CGContextRef context);
typedef void (*CGEColorReleaseUserPatternInfoCallback)(void* info);

typedef struct CGEColorFunctionCallbacks CGEColorFunctionCallbacks;

typedef void (*CGEColorFunctionReleaseInfoCallback)(void* info);
typedef size_t (*CGEColorFunctionGetNumberOfBaseColorsCallback)(void* info);
typedef const CGFloat* (*CGEColorFunctionGetBaseColorsComponentsCallback)(void* info);

// `functionCallbacksPtr' is in-out: on input it is base colors info callbacks record for `function', on output it is base colors info callbacks record for the result function. `*functionCallbacksPtr' may be set to NULL.
// Note: `(**functionCallbacksPtr).info' should be retained if left the same on output.
typedef CGFunctionRef (*CGEColorFunctionCreateCopyWithBaseColorsComponentsCallback)(void* info, const CGFloat* components, CGEColorFunctionCallbacks** functionCallbacksPtr);

// This structure is version 0.
struct CGEColorFunctionCallbacks
{
    size_t version;
    void* info;
    CGEColorFunctionReleaseInfoCallback releaseInfoCallback;
    CGEColorFunctionGetNumberOfBaseColorsCallback getNumberOfBaseColorsCallback;
    CGEColorFunctionGetBaseColorsComponentsCallback getBaseColorsComponentsCallback;
    CGEColorFunctionCreateCopyWithBaseColorsComponentsCallback createCopyWithBaseColorsComponentsCallback;
};

CG_EXTERN CGEColorRef CGEColorCreateWithCGColor(CGColorRef color);

CG_EXTERN CGEColorRef CGEColorCreateSolid(CGColorSpaceRef colorSpace, const CGFloat* components); // Creates CGColor-based solid color.

// `colorSpace' and `components' are color space and color components (including alpha) for stencil pattern.
// `colorSpace' may be NULL: colored pattern is created, `components' should contain one component in this case - alpha.
// `releaseUserPatternInfoCallback' may be NULL.
CG_EXTERN CGEColorRef CGEColorCreateUserPattern(void* info, CGRect bounds, CGFloat xStep, CGFloat yStep, CGPatternTiling tiling, CGColorSpaceRef colorSpace, const CGFloat* components, CGEColorDrawUserPatternCallback drawUserPaternCallback, CGEColorReleaseUserPatternInfoCallback releaseUserPatternInfoCallback);
CG_EXTERN CGEColorRef CGEColorCreateUserPatternWithImageCell(CGImageRef image, CGRect bounds, bool isFlippedHorizontally, bool isFlippedVertically, CGFloat xStep, CGFloat yStep, CGPatternTiling tiling, CGColorSpaceRef colorSpace, const CGFloat* components);
CG_EXTERN CGEColorRef CGEColorCreateUserPatternWithPDFPageCell(CGPDFDocumentRef document, size_t pageNumber, CGRect bounds, bool isFlippedHorizontally, bool isFlippedVertically, CGFloat xStep, CGFloat yStep, CGPatternTiling tiling, CGColorSpaceRef colorSpace, const CGFloat* components);
CG_EXTERN CGEColorRef CGEColorCreateUserPatternWithImage(CGImageRef image, bool isFlippedHorizontally, bool isFlippedVertically, CGPatternTiling tiling, CGColorSpaceRef colorSpace, const CGFloat* components);

// Gradient's geometry is in space {{0.0, 0.0}, {1.0, 1.0}}, which is mapped to color bounds during drawing.
// `functionCallbacks' may be NULL. `info' field & any callback in `functionCallbacks' may be NULL.
// Known issue (to be fixed later): rectangular gradient works correctly only if `startRect' is inside `endRect'.
CG_EXTERN CGEColorRef CGEColorCreateAxialGradient(CGColorSpaceRef colorSpace, CGPoint startPoint, CGPoint endPoint, CGFunctionRef function, const CGEColorFunctionCallbacks* functionCallbacks, bool extendStart, bool extendEnd);
CG_EXTERN CGEColorRef CGEColorCreateRadialGradient(CGColorSpaceRef colorSpace, CGPoint startPoint, CGFloat startRadius, CGPoint endPoint, CGFloat endRadius, CGAffineTransform transform, CGFunctionRef function, const CGEColorFunctionCallbacks* functionCallbacks, bool extendStart, bool extendEnd);
CG_EXTERN CGEColorRef CGEColorCreateRectangularGradient(CGColorSpaceRef colorSpace, CGRect startRect, CGRect endRect, CGAffineTransform transform, CGFunctionRef function, const CGEColorFunctionCallbacks* functionCallbacks, bool extendStart, bool extendEnd);

// Gradient's geometry is in space {{0.0, 0.0}, {1.0, 1.0}}, which is mapped to color bounds during drawing.
// `intermediatePoints' points to array of intermediate points in range (0.0, 1.0). `intermediatePoints' may be NULL: `numberOfIntermediatePoints' intervening colors will be at equal intervals in between 0.0 and 1.0.
// `components' is two-dimensional array of `numberOfIntermediatePoints' + 2 (one for start point and one for end point) colors, each color is array of N components, where N is one more (for alpha) than the number of color components in the gradient's colorspace.
// Known issue (to be fixed later): rectangular gradient works correctly only if `startRect' is inside `endRect'.
CG_EXTERN CGEColorRef CGEColorCreateLinearAxialGradient(CGColorSpaceRef colorSpace, CGPoint startPoint, CGPoint endPoint, size_t numberOfIntermediatePoints, const CGFloat* intermediatePoints, const CGFloat* components, bool extendStart, bool extendEnd);
CG_EXTERN CGEColorRef CGEColorCreateLinearRadialGradient(CGColorSpaceRef colorSpace, CGPoint startPoint, CGFloat startRadius, CGPoint endPoint, CGFloat endRadius, CGAffineTransform transform, size_t numberOfIntermediatePoints, const CGFloat* intermediatePoints, const CGFloat* components, bool extendStart, bool extendEnd);
CG_EXTERN CGEColorRef CGEColorCreateLinearRectangularGradient(CGColorSpaceRef colorSpace, CGRect startRect, CGRect endRect, CGAffineTransform transform, size_t numberOfIntermediatePoints, const CGFloat* intermediatePoints, const CGFloat* components, bool extendStart, bool extendEnd);

CG_EXTERN CGEColorRef CGEColorCreateCopyWithBaseColorsComponents(CGEColorRef color, const CGFloat* components);

CG_EXTERN CGEColorRef CGEColorRetain(CGEColorRef color);
CG_EXTERN void CGEColorRelease(CGEColorRef color);

CG_EXTERN bool CGEColorEqualToColor(CGEColorRef color1, CGEColorRef color2);

CG_EXTERN bool CGEColorIsSolid(CGEColorRef color);
CG_EXTERN bool CGEColorIsPattern(CGEColorRef color); // Returns TRUE if `color' is user pattern or is based on pattern CGColor.
CG_EXTERN bool CGEColorIsGradient(CGEColorRef color);

CG_EXTERN CGColorRef CGEColorGetColor(CGEColorRef color); // Returns underlying CGColor if `color' is CGColor-based, otherwise returns NULL.
CG_EXTERN bool CGEColorIsUserPattern(CGEColorRef color);

CG_EXTERN CGColorSpaceRef CGEColorGetBaseColorSpace(CGEColorRef color);

// These functions return information about base solid colors if available.
CG_EXTERN size_t CGEColorGetNumberOfBaseColors(CGEColorRef color);
CG_EXTERN const CGFloat* CGEColorGetBaseColorsComponents(CGEColorRef color);

// Note: alpha of a gradient color is always 1.0.
CG_EXTERN CGFloat CGEColorGetAlpha(CGEColorRef color);

// For gradient `color' `colorBounds' defines bounds. For user pattern `color' origin of `colorBounds' defines pattern origin (and size of `colorBounds' is not used). For CGColor-based `color' `colorBounds' are not used.
CG_EXTERN void CGEContextStrokePathWithColor(CGContextRef context, CGEColorRef color, CGRect colorBounds); // Stroking with gradient colors is not supported prior to Tiger.
CG_EXTERN void CGEContextFillPathWithColor(CGContextRef context, CGEColorRef color, CGRect colorBounds);
CG_EXTERN void CGEContextEOFillPathWithColor(CGContextRef context, CGEColorRef color, CGRect colorBounds);

CG_EXTERN_C_END

#endif // CGEColor_H_
