/*!
    @header CGEGeometry.h
    @project SMKCGExtensions
    @author Alexey Navrotsky
    @creation_date 01/12/2007
    @copyright Copyright (c) 2007 The Software MacKiev Company. All rights reserved.
*/

#ifndef CGEGeometry_H_
#define CGEGeometry_H_

#include <ApplicationServices/../Frameworks/CoreGraphics.framework/Headers/CGGeometry.h>
#include <SMKCGExtensions/CGEFloat.h>
#include <SMKCGExtensions/CGEAffineTransform.h>

CG_EXTERN_C_BEGIN

typedef enum
{
    kCGEPixelizationPointPositionClosest = 0, // Pixelization point is in source point's pixel corner which is the closest to the source point of primitive being pixelized.
    kCGEPixelizationPointPositionInner, // Pixelization point is in source point's pixel corner which is the closest to the end of primitive being pixelized.
    kCGEPixelizationPointPositionOuter, // Pixelization point is in source point's pixel corner which is the farthest from the end of primitive being pixelized.
    kCGEPixelizationPointPositionExternal // Pixelization point is specified externally.
} CGEPixelizationPointPosition;

typedef enum
{
    kCGEPolylinePointTypeStart = 0,
    kCGEPolylinePointTypeCorner,
    kCGEPolylinePointTypeEnd
} CGEPolylinePointType;

typedef void (*CGEPolylineConsumerFunction)(void* info, CGEPolylinePointType pointType, CGPoint point);

CG_INLINE CGRect CGERectAssemble(CGPoint origin, CGSize size);
CG_INLINE CGRect CGERectMakeWithCorners(CGPoint corner1, CGPoint corner2);
CG_INLINE CGRect CGERectMakeBounds(CGFloat minX, CGFloat minY, CGFloat maxX, CGFloat maxY);
CG_EXTERN CGRect CGERectMakeBoundsForPoints(const CGPoint* points, size_t count);

// `cornersPtr' should point to a memory block large enough to hold 4 points.
CG_INLINE void CGERectGetCorners(CGRect rect, CGPoint* cornersPtr);

CG_INLINE CGRect CGERectIncludePoint(CGRect rect, CGPoint point);

// `resultRectsPtr' should point to a memory block large enough to hold 4 rects.
// Returns whether input rects intersect.
CG_EXTERN bool CGERectSubtraction(CGRect rect1, CGRect rect2, CGRect* resultRectsPtr, size_t* resultRectCountPtr);

#if MAC_OS_X_VERSION_MIN_REQUIRED >= MAC_OS_X_VERSION_10_4
    #define CGERectApplyAffineTransform CGRectApplyAffineTransform
#else
    #define CGERectApplyAffineTransform __CGERectApplyAffineTransform
    
    CG_EXTERN CGRect CGERectApplyAffineTransform(CGRect rect, CGAffineTransform transform);
#endif // MAC_OS_X_VERSION_MIN_REQUIRED >= MAC_OS_X_VERSION_10_4

// If returned rect is transformed with `destinationTransform' and stroken with width `destinationLineWidth' then result drawing is bound by `rect' transformed with `destinationTransform'.
CG_INLINE CGRect CGERectInsetForStroke(CGRect rect, CGAffineTransform destinationTransform, CGFloat destinationLineWidth);

CG_INLINE CGRect CGERectCalculateShadowRect(CGRect rect, CGSize shadowOffset, CGFloat shadowBlurRadius);
CG_INLINE CGRect CGERectCalculateShadowMaskRect(CGRect rect, CGSize shadowOffset, CGSize shadowMaskSize);

// Gradient created with the result axis covers whole area of `rect'.
CG_INLINE void CGERectGetGradientAxisForAngle(CGRect rect, CGFloat angle, CGPoint* startPointPtr, CGPoint* endPointPtr);

// Pixelizes line from `point1' to `point2' with `pixelizationStep'.
// If `startPointPosition' is kCGEPixelizationPointPositionExternal, then `startPoint' specifies start point of pixelized polyline. Otherwise, `startPoint' is ignored.
// `endPointPosition' may not be kCGEPixelizationPointPositionExternal.
CG_EXTERN void CGEPolylinePixelizeLine(CGPoint point1, CGPoint point2, CGSize pixelizationStep, CGEPixelizationPointPosition startPointPosition, CGPoint startPoint, CGEPixelizationPointPosition endPointPosition, void* consumerInfo, CGEPolylineConsumerFunction consumerFunction);

#pragma mark -
#pragma mark *** Inline Definitions ***
#pragma mark -

CG_INLINE CGRect CGERectAssemble(CGPoint origin, CGSize size)
{
    CGRect result = {origin, size};
    
    return result;
}

CG_INLINE CGRect CGERectMakeBounds(CGFloat minX, CGFloat minY, CGFloat maxX, CGFloat maxY)
{
    return CGRectMake(minX, minY, maxX - minX, maxY - minY);
}

CG_INLINE CGRect CGERectMakeWithCorners(CGPoint corner1, CGPoint corner2)
{
    CGFloat minX = corner1.x;
    CGFloat maxX = corner1.x;
    CGFloat minY = corner1.y;
    CGFloat maxY = corner1.y;
    
    if (minX > corner2.x)
        minX = corner2.x;
    else maxX = corner2.x;
    
    if (minY > corner2.y)
        minY = corner2.y;
    else maxY = corner2.y;
    
    return CGERectMakeBounds(minX, minY, maxX, maxY);
}

CG_INLINE void CGERectGetCorners(CGRect rect, CGPoint* cornersPtr)
{
    cornersPtr[0] = rect.origin;
    cornersPtr[1] = CGPointMake(rect.origin.x, rect.origin.y + rect.size.height);
    cornersPtr[2] = CGPointMake(rect.origin.x + rect.size.width, cornersPtr[1].y);
    cornersPtr[3] = CGPointMake(cornersPtr[2].x, rect.origin.y);
}

CG_INLINE CGRect CGERectIncludePoint(CGRect rect, CGPoint point)
{
    if (point.x < rect.origin.x)
    {
        rect.size.width += rect.origin.x - point.x;
        rect.origin.x = point.x;
    }
    else if (point.x > (rect.origin.x + rect.size.width))
        rect.size.width = point.x - rect.origin.x;
    
    if (point.y < rect.origin.y)
    {
        rect.size.height += rect.origin.y - point.y;
        rect.origin.y = point.y;
    }
    else if (point.y > (rect.origin.y + rect.size.height))
        rect.size.height = point.y - rect.origin.y;
    
    return rect;
}

CG_INLINE CGRect CGERectInsetForStroke(CGRect rect, CGAffineTransform destinationTransform, CGFloat destinationLineWidth)
{
    CGFloat destinationInset = destinationLineWidth * 0.5;
    CGFloat widthScaleFactor = CGEAffineTransformGetWidthScaleFactor(destinationTransform);
    CGFloat heightScaleFactor = CGEAffineTransformGetHeightScaleFactor(destinationTransform);
    
    return CGRectInset(rect, (widthScaleFactor == 0.0) ? 0.0 : destinationInset / widthScaleFactor, (heightScaleFactor == 0.0) ? 0.0 : destinationInset / heightScaleFactor);
}

CG_INLINE CGRect CGERectCalculateShadowRect(CGRect rect, CGSize shadowOffset, CGFloat shadowBlurRadius)
{
    return CGRectOffset(CGRectInset(rect, -shadowBlurRadius, -shadowBlurRadius), shadowOffset.width, shadowOffset.height);
}

CG_INLINE CGRect CGERectCalculateShadowMaskRect(CGRect rect, CGSize shadowOffset, CGSize shadowMaskSize)
{
    return CGRectMake(
        rect.origin.x + (rect.size.width - shadowMaskSize.width) * 0.5 + shadowOffset.width,
        rect.origin.y + (rect.size.height - shadowMaskSize.height) * 0.5 + shadowOffset.height,
        shadowMaskSize.width,
        shadowMaskSize.height
    );
}

CG_INLINE void CGERectGetGradientAxisForAngle(CGRect rect, CGFloat angle, CGPoint* startPointPtr, CGPoint* endPointPtr)
{
    // Make `angle' to be in range [0, 2pi).
    angle = CGEFloat_mod(angle, 2.0 * M_PI);
    if (angle < 0.0)
        angle += 2.0 * M_PI;
    
    if (angle >= M_PI)
    { // Invert axis direction.
        CGPoint* temp = startPointPtr;
        startPointPtr = endPointPtr;
        endPointPtr = temp;
    }
    
    CGFloat sine = CGEFloat_sin(angle);
    CGFloat cosine = CGEFloat_cos(angle);
    
    if ((angle < M_PI_2) || ((angle >= M_PI) && (angle < 3.0 * M_PI_2)))
    {
        CGFloat temp = rect.size.height * sine + rect.size.width * cosine;
        
        startPointPtr->x = rect.origin.x;
        startPointPtr->y = rect.origin.y;
        endPointPtr->x = startPointPtr->x + temp * cosine;
        endPointPtr->y = startPointPtr->y + temp * sine;
    }
    else
    {
        CGFloat temp = rect.size.height * sine - rect.size.width * cosine;
        
        startPointPtr->x = rect.origin.x + rect.size.width;
        startPointPtr->y = rect.origin.y;
        endPointPtr->x = startPointPtr->x + temp * cosine;
        endPointPtr->y = startPointPtr->y + temp * sine;
    }
}

CG_EXTERN_C_END

#endif // CGEGeometry_H_
