/*!
    @header SMKSubmitInfo.h
    @project SMKSubmitInfo
    @author Michael Panchenko
    @creation_date 04/28/2004
    @copyright Copyright (c) 2005 The Software MacKiev Company. All rights reserved.
    @abstract Framework interface declaration.
    @discussion Framework provides API for collecting information and sending it through network using receiver URL. All functions currently is not thread-safe so they should be called from a main thread only. Information for submit should be represented by dictionary object. Constants below (SIFirstNameKey, SILastNameKey etc.) or any other strings can be used as keys for such dictionary. Strings only can be used as values for those keys so even user information dictionary returned by corresponding function below should be re-formated before using. Request encoding data is serialized dictionary that represents application specific information to create request. It should contains URL to postman submit form, path template for queue storage file and dictionary with [submit info key]-[postman form key] map. Values from submit dictionary will be added to a request only when the same key present in encoding keys map dictionary.
*/

#if !defined(SMKSUBMITINFO_H)
#   define SMKSUBMITINFO_H

#include <CoreFoundation/CoreFoundation.h>
#include <SMKSubmitInfo/SMKSubmitInfoTypes.h>


/*!
    @defined    SIFirstNameKey
    @abstract   User first name key.
    @discussion Value type: CFStringRef
*/
#define SIFirstNameKey      CFSTR("fname")

/*!
    @defined    SILastNameKey
    @abstract   User last name key.
    @discussion Value type: CFStringRef
*/
#define SILastNameKey       CFSTR("lname")

/*!
    @defined    SIEmailKey
    @abstract   User email address key.
    @discussion Value type: CFStringRef
*/
#define SIEmailKey          CFSTR("email")

/*!
    @defined    SICityKey
    @abstract   User city name key.
    @discussion Value type: CFStringRef
*/
#define SICityKey           CFSTR("city")

/*!
    @defined    SIStateKey
    @abstract   User state name key.
    @discussion Value type: CFStringRef
*/
#define SIStateKey          CFSTR("state")

/*!
    @defined    SICountryKey
    @abstract   User country key.
    @discussion Value type: CFStringRef
*/
#define SICountryKey        CFSTR("country")

/*!
    @defined    SIPatformKey
    @abstract   Platform type (PPC, Intel itc).
    @discussion Value type: CFNumberRef (identifier returned by Gestalt() function for gestaltSysArchitecture selector)
*/
#define SIPatformKey        CFSTR("platform")

/*!
    @defined    SIOSVersionKey
    @abstract   OS Version key.
    @discussion Value type: CFStringRef
*/
#define SIOSVersionKey      CFSTR("os version")



SI_EXTERN_C_BEGIN

/*!
    @function   SICreateUserInformation
    @abstract   Get user information from corresponding Address Book record.
    @result     User information dictionary. Ownership follows the Create Rule.
*/
extern CFDictionaryRef SICreateUserInformation();

/*!
    @function   SIGetRequestsInQueueCount
    @abstract   Get count of requests in queue.
    @result     Requests count.
*/
extern UInt32 SIGetRequestsInQueueCount();

/*!
    @function   SIClearRequestsQueue
    @abstract   Remove all requests from queue.
*/
extern void SIClearRequestsQueue();

/*!
    @function   SIClearRequestsQueueWithIdentifier
    @abstract   Remove sender-specific requests from queue.
    @param      applicationIdentifier application (sender) identifier whose requests should be removed from queue (pass NULL to remove all requests)
*/
extern void SIClearRequestsQueueWithIdentifier(CFStringRef applicationIdentifier);

/*!
    @function   SISetRequestsQueueLimit
    @abstract   Set maximal active requests count in queue.
    @param      limit maximal active requests count (queue will be unlimited when 0 value specified)
*/
extern void SISetRequestsQueueLimit(UInt32 limit);

/*!
    @function   SIGetRequestsQueueLimit
    @abstract   Get maximal active requests count in queue.
    @result     Maximal active requests count (0 means unlimited requests queue)
*/
extern UInt32 SIGetRequestsQueueLimit();

/*!
    @function   SIStoreUnsentRequests
    @abstract   Store unsent requests from queue to the local disk to try send them again on next time (best time to call is on application termination)
    @param      applicationIdentifier application identifier whose requests should be stored (pass NULL to store whole requests)
    @result     true when operation completed successfully.
*/
extern Boolean SIStoreUnsentRequests(CFStringRef applicationIdentifier);

/*!
    @function   SIIsSubmitInformationSizeAcceptable
    @abstract   Check submit information size.
    @discussion Function designed for debug mode but it is "too heavy" for release one
    @param      information information to submit (cannot be NULL)
    @param      encodingData request encoding data (cannot be NULL)
    @result     Checking result.
*/
extern Boolean SIIsSubmitInformationSizeAcceptable(CFDictionaryRef information, CFDataRef encodingData);

/*!
    @function   SISubmitInformationFirst
    @abstract   Create and add new request to the submit queue head (to send first).
    @param      information information to submit (specify NULL to load requests not sent from previous session)
    @param      encodingData request encoding data (cannot be NULL)
    @param      applicationVersion application version as a string (value for kCFBundleVersionKey from main bundle info dictionary will be used when NULL specified)
    @param      applicationIdentifier application bundle identifier (main bundle identifier will be used when NULL specified)
    @param      overflowFlags what to do when requests queue is overflow
    @result     Operation status.
*/
extern SIStatus SISubmitInformationFirst(CFDictionaryRef information, CFDataRef encodingData, CFStringRef applicationVersion, CFStringRef applicationIdentifier, SIOverflowFlags overflowFlags);

/*!
    @function   SISubmitInformationLast
    @abstract   Create and add new request to the submit queue tail (to send last).
    @param      information information to submit (specify NULL to load requests not sent from previous session)
    @param      encodingData request encoding data (cannot be NULL)
    @param      applicationVersion application version as a string (value for kCFBundleVersionKey from main bundle info dictionary will be used when NULL specified)
    @param      applicationIdentifier application bundle identifier (main bundle identifier will be used when NULL specified)
    @param      overflowFlags what to do when requests queue is overflow
    @result     Operation status.
*/
extern SIStatus SISubmitInformationLast(CFDictionaryRef information, CFDataRef encodingData, CFStringRef applicationVersion, CFStringRef applicationIdentifier, SIOverflowFlags overflowFlags);

SI_EXTERN_C_END

#endif // !defined(SMKSUBMITINFO_H)
